@@ -54,8 +54,8 @@ static void config_usage(int rw)
 	{
 		fprintf(stderr, " -o --outputfile    Name of output file (defaults to same as --configfile)\n");
 		fprintf(stderr, " -C --no_ccs        Don't tell CCSD about this change\n");
-		fprintf(stderr, "                    default: run \"ccs_tool update\" if file is updated in place)\n");
-		fprintf(stderr, " -F --force_ccs     Force \"ccs_tool upgrade\" even if input & output files differ\n");
+		fprintf(stderr, "                    (default: run \"ccs_tool update\" if file is updated in place)\n");
+		fprintf(stderr, " -F --force_ccs     Force \"ccs_tool update\" even if input & output files differ\n");
 	}
 }
 
