@@ -12,6 +12,7 @@
 ******************************************************************************/
 
 #include <unistd.h>
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -37,7 +38,7 @@
 #define FALSE 0
 #endif
 
-#define OPTION_STRING			("VhScj:f:DwQ")
+#define OPTION_STRING			("VhScj:f:t:DwQ")
 #define LOCKFILE_NAME                   "/var/run/fenced.pid"
 #define FENCED_SOCK_PATH                "fenced_socket"
 
@@ -61,6 +62,8 @@ int debug;
 int operation;
 int child_wait;
 int quorum_wait = TRUE;
+int fenced_start_timeout = 0;
+int signalled = 0;
 int cl_sock;
 char our_name[MAX_CLUSTER_MEMBER_NAME_LEN+1];
 
@@ -86,6 +89,27 @@ static int check_mounted(void)
 	return 0;
 }
 
+static void sigalarm_handler(int sig)
+{
+	signalled = 1;
+}
+
+static int get_int_arg(char argopt, char *arg)
+{
+        char *tmp;
+        int val;                                                                                 
+        val = strtol(arg, &tmp, 10);
+        if (tmp == arg || tmp != arg + strlen(arg))
+                die("argument to %c (%s) is not an integer", argopt, arg);
+                                                                                
+        if (val < 0)
+                die("argument to %c cannot be negative", argopt);
+                                                                                
+        return val;
+}
+                                                                                
+
+
 static void lockfile(void)
 {
 	int fd, error;
@@ -185,7 +209,7 @@ static int check_quorum(void)
 	if (debug)
 		printf("%s: wait for quorum %d\n", prog_name, quorum_wait);
 
-	while (1) {
+	while (!signalled) {
 		rv = ioctl(cl_sock, SIOCCLUSTER_ISACTIVE, NULL);
 		if (!rv)
 			die("cluster is not active");
@@ -198,9 +222,12 @@ static int check_quorum(void)
 
 		sleep(1);
 
-		if (++i > 9 && !(i % 10))
+		if (!signalled && ++i > 9 && !(i % 10))
 			printf("%s: waiting for cluster quorum\n", prog_name);
 	}
+
+	errno = ETIMEDOUT;
+	return FALSE;
 }
 
 /*
@@ -219,12 +246,12 @@ static int do_wait(void)
 	if (!file)
 		return EXIT_FAILURE;
 
-	while (1) {
+	while (!signalled) {
 		memset(line, 0, 256);
 		while (fgets(line, 256, file)) {
 			if (strstr(line, "Fence Domain")) {
 				if (strstr(line, "run")) {
-					error = 0;
+					error = EXIT_SUCCESS;
 					goto out;
 				}
 				starting = 1;
@@ -241,10 +268,12 @@ static int do_wait(void)
 		sleep(1);
 		rewind(file);
 	}
+	errno = ETIMEDOUT;
+	error = EXIT_FAILURE;
                         
  out:
 	fclose(file);
-	return EXIT_SUCCESS;
+	return error;
 }
 
 static int do_join(int argc, char *argv[])
@@ -253,8 +282,17 @@ static int do_join(int argc, char *argv[])
 
 	setup_sock();
 
-	if (!check_quorum())
+       	if (fenced_start_timeout) {
+               	signal(SIGALRM, sigalarm_handler);
+               	alarm(fenced_start_timeout);
+       	}
+
+	if (!check_quorum()) {
+		if (errno == ETIMEDOUT)
+			printf("%s: Timed out waiting for cluster "
+			       "quorum to form.\n", prog_name);
 		return EXIT_FAILURE;
+	}
 
 	get_our_name();
 	close(cl_sock);
@@ -275,12 +313,33 @@ static int do_join(int argc, char *argv[])
 	if (!debug && child_wait) {
 		int status;
 		pid_t pid = fork();
+
 		/* parent waits for fenced to join */
 		if (pid > 0) {
 			waitpid(pid, &status, 0);
-			if (WIFEXITED(status) && !WEXITSTATUS(status))
-				do_wait();
-			exit(EXIT_SUCCESS);
+			if (WIFEXITED(status) &&
+			    !WEXITSTATUS(status)) {
+				if (do_wait() == EXIT_SUCCESS)
+					exit(EXIT_SUCCESS);
+
+				if (errno == ETIMEDOUT)
+					printf("%s: Timed out waiting "
+					       "for fenced to start.\n",
+					       prog_name);
+				return EXIT_FAILURE;
+			}
+
+			/* If we get here, the child died with a signal */
+			if (WIFSIGNALED(status)) {
+				printf("%s: child pid %d died with "
+				       "signal %d\n", prog_name, (int)pid,
+				       WTERMSIG(status));
+			}
+
+			return EXIT_FAILURE;
+		} else if (pid < 0) {
+			fprintf(stderr, "%s: fork(): %s\n",
+				prog_name, strerror(errno));
 		}
 		/* child execs fenced */
 	}
@@ -371,6 +430,7 @@ static void print_usage(void)
 	printf("  -w               Wait for join to complete\n");
 	printf("  -V               Print program version information, then exit\n");
 	printf("  -h               Print this help, then exit\n");
+        printf("  -t               Maximum time in seconds to wait\n");
 	printf("  -Q               Fail if cluster is not quorate, don't wait\n");
 	printf("  -D               Enable debugging, don't fork (also passed to fenced)\n");
 	printf("\n");
@@ -426,6 +486,10 @@ static void decode_arguments(int argc, char *argv[])
 			cont = FALSE;
 			break;
 
+                case 't':
+                        fenced_start_timeout = get_int_arg(optchar, optarg);
+                        break;
+
 		case 'c':
 		case 'j':
 		case 'f':
