@@ -1465,12 +1465,13 @@ gfs_lock(struct file *file, int cmd, struct file_lock *fl)
 
 	if (sdp->sd_args.ar_localflocks) {
 		if (IS_GETLK(cmd)) {
-			struct file_lock *tmp;
+			struct file_lock tmp;
+			int ret;
 			lock_kernel();
-			tmp = posix_test_lock(file, fl);
+			ret = posix_test_lock(file, fl, &tmp);
 			fl->fl_type = F_UNLCK;
-			if (tmp)
-				memcpy(fl, tmp, sizeof(struct file_lock));
+			if (ret)
+				memcpy(fl, &tmp, sizeof(struct file_lock));
 			unlock_kernel();
 		        return 0;
 		} else {
