@@ -24,6 +24,8 @@
 #include <linux/mm.h>
 #include <linux/xattr.h>
 #include <linux/posix_acl.h>
+#include <net/sock.h>
+#include <cluster/cnxman.h>
 
 #include "gfs.h"
 #include "acl.h"
@@ -120,6 +122,33 @@ gfs_create(struct inode *dir, struct dentry *dentry,
 	return 0;
 }
 
+/** get_my_node_id - Returns the node id of the local node
+ *
+ *  Returns: the nodeid as int. If we are not part of a cluster
+ *          return 0.
+ */
+static int get_my_nodeid(void) 
+{
+	/* we want to keep the information. */
+ 	static struct kcl_cluster_node *us=NULL;
+	
+	/* if we already have our information, return it.*/
+	if (us) 
+		return us->node_id;
+	
+	/* we are here, i.e. we need to collect information */
+	us = kmalloc(sizeof (struct kcl_cluster_node), GFP_KERNEL);
+	if (kcl_get_node_by_nodeid(0, us)) 
+	{
+		/* request failed. Cleaning up. Returning 0.*/
+		kfree(us);
+		us = NULL;
+		return 0;
+	}
+	/* return collected data */
+	return us->node_id;			
+}
+
 /**
  * lookup_cdpn_sub_at - Maybe lookup a Context Dependent Pathname
  * @sdp: the filesystem
@@ -145,6 +174,11 @@ lookup_cdpn_sub_at(struct gfs_sbd *sdp, struct dentry *dentry)
 		new = lookup_one_len(system_utsname.nodename,
 				     parent,
 				     strlen(system_utsname.nodename));
+	else if (gfs_filecmp(&dentry->d_name, "@nodeid", 7))
+		new = lookup_one_len(buf, 
+				     parent, 
+				     sprintf(buf, "%s%i", "node", 
+					     get_my_nodeid()));
 	else if (gfs_filecmp(&dentry->d_name, "@mach", 5))
 		new = lookup_one_len(system_utsname.machine,
 				     parent,
@@ -204,6 +238,11 @@ lookup_cdpn_sub_brace(struct gfs_sbd *sdp, struct dentry *dentry)
 		new = lookup_one_len(system_utsname.nodename,
 				     parent,
 				     strlen(system_utsname.nodename));
+	else if (gfs_filecmp(&dentry->d_name, "{nodeid}", 8))
+		new = lookup_one_len(buf,
+				     parent,
+				     sprintf(buf, "%s%i", "node",
+					     get_my_nodeid()));
 	else if (gfs_filecmp(&dentry->d_name, "{mach}", 6))
 		new = lookup_one_len(system_utsname.machine,
 				     parent,
