@@ -29,7 +29,7 @@
 #include <linux/in.h>
 #include <linux/buffer_head.h>
 #include <linux/miscdevice.h>
-
+#include <linux/moduleparam.h>
 #include <linux/devfs_fs_kernel.h>
 
 #include <asm/uaccess.h>
@@ -1048,6 +1048,6 @@ MODULE_DESCRIPTION("Network Block Device");
 MODULE_LICENSE("GPL");
 
 #ifndef NDEBUG
-MODULE_PARM(debugflags, "i");
+module_param(debugflags, uint, 0644);
 MODULE_PARM_DESC(debugflags, "flags for controlling debug output");
 #endif
