@@ -515,6 +515,7 @@ int open_lt_to_core(void)
    uint64_t x_gen;
    uint32_t x_code, x_error, x_rank;
    uint8_t x_ama;
+   int connection_attempts = 0;
 
    if((cfd = socket(AF_INET6, SOCK_STREAM, 0)) <0) {
       log_err("Failed to create socket. %d:%s\n", errno, strerror(errno));
@@ -526,10 +527,19 @@ int open_lt_to_core(void)
    adr.sin6_addr = in6addr_loopback;
    adr.sin6_port = htons(gulm_config.corePort);
 
-   if( connect(cfd, (struct sockaddr*)&adr, sizeof(struct sockaddr_in6))<0) {
+   while ( connect(cfd, (struct sockaddr*)&adr, sizeof(struct sockaddr_in6))<0) {
       close(cfd);
-      log_err("Failed to connect to core. %d:%s\n", errno, strerror(errno));
-      return -1;
+
+      connection_attempts++;
+      if (connection_attempts > 3) {
+         log_err("Failed to connect to core, shutting down lock_gulmd_LT.");
+         return -1;
+      }
+      if (connection_attempts > 1)
+         log_err("Failed to connect to core. %d:%s\n", errno, strerror(errno));
+
+      sleep(5);
+
    }
 
    idx = add_to_pollers(cfd, poll_Open, 0, "_ core _", &in6addr_loopback);
