@@ -34,8 +34,12 @@
 #include <rg_queue.h>
 #include <malloc.h>
 
+#define L_SYS (1<<1)
+#define L_USER (1<<0)
+
 int configure_logging(int ccsfd);
 
+void node_event_q(int, uint64_t, int);
 int daemon_init(char *);
 int init_resource_groups(int);
 void kill_resource_groups(void);
@@ -44,9 +48,10 @@ int eval_groups(int, uint64_t, int);
 void graceful_exit(int);
 void flag_shutdown(int sig);
 void hard_exit(void);
-int send_rg_states(int);
+int send_rg_states(int, int);
 int check_config_update(void);
 int svc_exists(char *);
+int watchdog_init(void);
 
 int shutdown_pending = 0, running = 1, need_reconfigure = 0;
 char debug = 0; /* XXX* */
@@ -134,6 +139,9 @@ flag_reconfigure(int sig)
 void
 node_event(int local, uint64_t nodeID, int nodeStatus)
 {
+	if (!running)
+		return;
+
 	if (local) {
 
 		/* Local Node Event */
@@ -216,8 +224,15 @@ membership_update(void)
 
 		clulog(LOG_INFO, "State change: %s DOWN\n",
 		       node_delta->cml_members[x].cm_name);
-		node_event(0, node_delta->cml_members[x].cm_id,
-			   STATE_DOWN);
+		/* Don't bother evaluating anything resource groups are
+		   locked.  This is just a performance thing */
+		if (!rg_locked()) {
+			node_event_q(0, node_delta->cml_members[x].cm_id,
+			     		STATE_DOWN);
+		} else {
+			clulog(LOG_NOTICE, "Not taking action - services"
+			       " locked\n");
+		}
 	}
 
 	/* Free nodes */
@@ -231,7 +246,7 @@ membership_update(void)
 	me = memb_online(node_delta, my_id());
 	if (me) {
 		clulog(LOG_INFO, "State change: Local UP\n");
-		node_event(1, my_id(), STATE_UP);
+		node_event_q(1, my_id(), STATE_UP);
 	}
 
 	for (x=0; node_delta && x < node_delta->cml_count; x++) {
@@ -245,14 +260,69 @@ membership_update(void)
 
 		clulog(LOG_INFO, "State change: %s UP\n",
 		       node_delta->cml_members[x].cm_name);
-		node_event(0, node_delta->cml_members[x].cm_id,
-			   STATE_UP);
+		node_event_q(0, node_delta->cml_members[x].cm_id,
+			     STATE_UP);
 	}
 
 	cml_free(node_delta);
 	cml_free(new_ml);
 
-	rg_unlockall();
+	rg_unlockall(L_SYS);
+
+	return 0;
+}
+
+
+int
+lock_commit_cb(char *key, uint64_t viewno, void *data, uint32_t datalen)
+{
+	char lockstate;
+
+	if (datalen != 1) {
+		clulog(LOG_WARNING, "%s: invalid data length!\n", __FUNCTION__);
+		free(data);
+		return 0;
+	}
+
+       	lockstate = *(char *)data;
+	free(data);
+
+	if (lockstate == 0) {
+		rg_unlockall(L_USER); /* Doing this multiple times
+					 has no effect */
+		clulog(LOG_NOTICE, "Resource Groups Unlocked\n");
+		return 0;
+	}
+
+	if (lockstate == 1) {
+		rg_lockall(L_USER); /* Doing this multiple times
+				       has no effect */
+		clulog(LOG_NOTICE, "Resource Groups Locked\n");
+		return 0;
+	}
+
+	clulog(LOG_DEBUG, "Invalid lock state in callback: %d\n", lockstate);
+	return 0;
+}
+
+
+int
+do_lockreq(int fd, int req)
+{
+	int ret;
+	char state;
+	cluster_member_list_t *m = member_list();
+
+	state = (req==RG_LOCK)?1:0;
+	ret = vf_write(m, VFF_IGN_CONN_ERRORS, "rg_lockdown", &state, 1);
+	cml_free(m);
+
+	if (ret == 0) {
+		msg_send_simple(fd, RG_SUCCESS, 0, 0);
+	} else {
+		msg_send_simple(fd, RG_FAIL, 0, 0);
+	}
+
 	return 0;
 }
 
@@ -292,9 +362,38 @@ dispatch_msg(int fd, uint64_t nodeid)
 	switch (msg_hdr.gh_command) {
 	case RG_STATUS:
 		clulog(LOG_DEBUG, "Sending service states to fd%d\n",fd);
-		send_rg_states(fd);
+		send_rg_states(fd, msg_hdr.gh_arg1);
 		break;
 
+
+	case RG_LOCK:
+		msg_receive_timeout(fd, &msg_hdr, sizeof(msg_hdr), 1);
+		if (rg_quorate()) {
+			do_lockreq(fd, RG_LOCK);
+		}
+
+		msg_close(fd);
+		break;
+
+	case RG_UNLOCK:
+		msg_receive_timeout(fd, &msg_hdr, sizeof(msg_hdr), 1);
+		if (rg_quorate()) {
+			do_lockreq(fd, RG_UNLOCK);
+		}
+
+		msg_close(fd);
+		break;
+
+	case RG_QUERY_LOCK:
+		msg_receive_timeout(fd, &msg_hdr, sizeof(msg_hdr), 1);
+		if (rg_quorate()) {
+			ret = (rg_locked() & L_USER) ? RG_LOCK : RG_UNLOCK;
+			msg_send_simple(fd, ret, 0, 0);
+		}
+		msg_close(fd);
+		break;
+
+
 	case RG_ACTION_REQUEST:
 
 		ret = msg_receive_timeout(fd, &msg_sm, sizeof(msg_sm), 
@@ -308,17 +407,6 @@ dispatch_msg(int fd, uint64_t nodeid)
 		/* Decode SmMessageSt message */
 		swab_SmMessageSt(&msg_sm);
 
-		if (rg_locked()) {
-			msg_sm.sm_data.d_ret = RG_EAGAIN;
-			/* Encode before responding... */
-			swab_SmMessageSt(&msg_sm);
-
-			if (msg_send(fd, &msg_sm, sizeof (SmMessageSt)) !=
-		    	    sizeof (SmMessageSt))
-				clulog(LOG_ERR, "#40: Error replying to "
-				       "action request.\n");
-		}
-
 		if (!svc_exists(msg_sm.sm_data.d_svcName)) {
 			msg_sm.sm_data.d_ret = RG_ENOSERVICE;
 			/* No such service! */
@@ -375,18 +463,19 @@ handle_cluster_event(int fd)
 		break;
 	case CE_SUSPEND:
 		clulog(LOG_DEBUG, "Suspend Event\n");
-		rg_lockall();
+		rg_lockall(L_SYS);
 		break;
 	case CE_MEMB_CHANGE:
 		clulog(LOG_DEBUG, "Membership Change Event\n");
-		if (rg_quorate()) {
-			rg_unlockall();
+		if (rg_quorate() && running) {
+			rg_unlockall(L_SYS);
 			membership_update();
 		}
 		break;
 	case CE_QUORATE:
 		rg_set_quorate();
-		rg_unlockall();
+		rg_unlockall(L_SYS);
+		rg_unlockall(L_USER);
 		clulog(LOG_NOTICE, "Quorum Achieved\n");
 		membership_update();
 		break;
@@ -394,7 +483,7 @@ handle_cluster_event(int fd)
 		clulog(LOG_EMERG, "#1: Quorum Dissolved\n");
 		rg_set_inquorate();
 		member_list_update(NULL);		/* Clear member list */
-		rg_lockall();
+		rg_lockall(L_SYS);
 		rg_doall(RG_INIT, 1, "Emergency stop of %s");
 		rg_set_uninitialized();
 		break;
@@ -430,7 +519,7 @@ event_loop(int clusterfd)
 		 */
 	}
 
-	while (tv.tv_sec || tv.tv_usec) {
+	while (running && (tv.tv_sec || tv.tv_usec)) {
 		FD_ZERO(&rfds);
 		max = msg_fill_fdset(&rfds, MSG_LISTEN, RG_PURPOSE);
 		FD_SET(clusterfd, &rfds);
@@ -489,8 +578,6 @@ event_loop(int clusterfd)
 		return 0;
 	}
 
-
-
 	return 0;
 }
 
@@ -512,7 +599,7 @@ graceful_exit(int sig)
 void
 hard_exit(void)
 {
-	rg_lockall();
+	rg_lockall(L_SYS);
 	rg_doall(RG_INIT, 1, "Emergency stop of %s");
 	vf_shutdown();
 	exit(1);
@@ -522,8 +609,8 @@ hard_exit(void)
 void
 cleanup(int cluster_fd)
 {
-	rg_lockall();
-	rg_doall(RG_STOP, 1, NULL);
+	rg_lockall(L_SYS);
+	rg_doall(RG_STOP_EXITING, 1, NULL);
 	vf_shutdown();
 	kill_resource_groups();
 	member_list_update(NULL);
@@ -648,8 +735,11 @@ main(int argc, char **argv)
 	if (foreground)
 		clu_log_console(1);
 
-	if (!foreground && (geteuid() == 0)) 
+	if (!foreground && (geteuid() == 0)) {
 		daemon_init(argv[0]);
+		if (!debug && !watchdog_init())
+			clulog(LOG_NOTICE, "Failed to start watchdog\n");
+	}
 
 	/*
 	   We need quorum before we can read the configuration data from
@@ -723,6 +813,8 @@ main(int argc, char **argv)
 		return -1;
 	}
 
+	vf_key_init("rg_lockdown", 10, NULL, lock_commit_cb);
+
 	if (clu_login(cluster_fd, RG_SERVICE_GROUP) == -1) {
 		if (errno != ENOSYS) {
 			clu_log_console(1);
@@ -756,7 +848,6 @@ main(int argc, char **argv)
 	
 	/*malloc_dump_table(); */ /* Only works if alloc.c us used */
 	/*malloc_stats();*/
-	/*malloc_dump_table(1352, 1352);*/
 
 	exit(0);
 }
